package com.bpcl.lpg.core.service.impl;

import org.apache.logging.log4j.LogManager;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.util.Objects;
import java.text.MessageFormat;
import java.net.http.HttpResponse;
import com.bpcl.lpg.core.model.LPGDistributorStaffModel;
import com.bpcl.core.util.BPCLDateUtils;
import java.util.Date;
import com.bpcl.lpg.core.enums.GenericStatusEnum;
import org.apache.commons.lang3.StringUtils;
import com.bpcl.lpg.core.model.OperatorCODRefillsModel;
import com.bpcl.lpg.facades.data.LpgRefillBookingRequestData;
import com.bpcl.lpg.core.pojo.RspResponseData;
import com.bpcl.lpg.core.service.BpclEkycService;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import com.bpcl.lpg.core.encryption.LpgEncryptionService;
import com.bpcl.lpg.core.credentials.LPGCredentialsService;
import javax.annotation.Resource;
import de.hybris.platform.servicelayer.keygenerator.KeyGenerator;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.model.ModelService;
import org.apache.logging.log4j.Logger;
import com.bpcl.lpg.core.service.LpgOperatorService;

public class LpgOperatorServiceImpl implements LpgOperatorService
{
    private static final Logger LOG;
    private static final String REFILL_BOOKING_END_POINT = "operatorRefillBookingUrl";
    private ModelService modelService;
    private ConfigurationService configurationService;
    @Resource
    private KeyGenerator operatorRefillKeyGenerator;
    @Resource
    private KeyGenerator operatorUniqueApiKeyGenerator;
    private LPGCredentialsService lpgCredentialsService;
    private LpgEncryptionService lpgEncryptionService;
    private BpclHttpClientService bpclHttpClientService;
    private BpclEkycService bpclEkycService;
    
    public void createOperatorRefill(final RspResponseData refillResponseData, final LpgRefillBookingRequestData refillRequestData, final String operatorMobile, final String correlationId, final OperatorCODRefillsModel operatorCodRefillBookingModel) {
        if (StringUtils.isNotBlank(refillRequestData.getConsumerNo())) {
            operatorCodRefillBookingModel.setConsumerNo(refillRequestData.getConsumerNo());
        }
        if (StringUtils.isNotBlank(refillRequestData.getLpgId())) {
            operatorCodRefillBookingModel.setLpgID(refillRequestData.getLpgId());
        }
        if (StringUtils.isNotBlank(refillRequestData.getContactNumber())) {
            operatorCodRefillBookingModel.setCustomerMobile(refillRequestData.getContactNumber());
        }
        if (StringUtils.isNotBlank(refillRequestData.getDistCode())) {
            operatorCodRefillBookingModel.setDistCode(refillRequestData.getDistCode());
        }
        if (StringUtils.isNotBlank(refillRequestData.getPartnerTransId())) {
            operatorCodRefillBookingModel.setPartnerTransId(refillRequestData.getPartnerTransId());
        }
        if (StringUtils.isNotBlank(refillRequestData.getUniqueAPIReqNo())) {
            operatorCodRefillBookingModel.setUniqueApiReqNo(refillRequestData.getUniqueAPIReqNo());
        }
        if (StringUtils.isNotBlank(refillResponseData.getOrderNo())) {
            operatorCodRefillBookingModel.setOrderNo(refillResponseData.getOrderNo());
        }
        operatorCodRefillBookingModel.setStatus(this.getStatus(refillResponseData.getReturnCode()));
        operatorCodRefillBookingModel.setCorrelationId(correlationId);
        if (StringUtils.isNotBlank(refillResponseData.getProductQty())) {
            operatorCodRefillBookingModel.setQuantity(Integer.valueOf(Integer.parseInt(refillResponseData.getProductQty())));
        }
        if (StringUtils.isNotBlank(refillResponseData.getBpclReferenceId())) {
            operatorCodRefillBookingModel.setBpclReferenceId(refillResponseData.getBpclReferenceId());
        }
        if (StringUtils.isNotBlank(refillResponseData.getProductType())) {
            operatorCodRefillBookingModel.setProductCode(refillResponseData.getProductType());
        }
        operatorCodRefillBookingModel.setBookDate(this.getBookingDate(refillResponseData));
        if (StringUtils.isNotBlank(refillResponseData.getAmount())) {
            operatorCodRefillBookingModel.setAmount(Double.valueOf(refillResponseData.getAmount()));
        }
        operatorCodRefillBookingModel.setOperatorMobile(operatorMobile);
        if (StringUtils.isNotBlank(refillResponseData.getReturnCode())) {
            operatorCodRefillBookingModel.setReturnCode(refillResponseData.getReturnCode());
        }
        if (StringUtils.isNotBlank(refillResponseData.getReturnMsg())) {
            operatorCodRefillBookingModel.setReturnMsg(refillResponseData.getReturnMsg());
        }
        this.modelService.save((Object)operatorCodRefillBookingModel);
    }
    
    private GenericStatusEnum getStatus(final String returnCode) {
        return "200".equals(returnCode) ? GenericStatusEnum.SUCCESS : GenericStatusEnum.FAILED;
    }
    
    private Date getBookingDate(final RspResponseData refillResponseData) {
        final String bookingDate = refillResponseData.getBookDate();
        if (org.apache.commons.lang.StringUtils.equalsIgnoreCase(refillResponseData.getReturnCode(), "200")) {
            return BPCLDateUtils.getDateAsSpecifiedFormat(bookingDate, "MMM dd yyyy hh:mma");
        }
        if (org.apache.commons.lang.StringUtils.equalsIgnoreCase(refillResponseData.getReturnCode(), "201")) {
            return BPCLDateUtils.getDateAsSpecifiedFormat(bookingDate, "yyyy-MM-dd hh:mm:ss.sss");
        }
        return null;
    }
    
    public void populateRefillBookingRequestData(final LpgRefillBookingRequestData operatorRefillRequestData, final String operatorMobile) {
        final String operatorID = "op".concat(operatorMobile);
        final LPGDistributorStaffModel operatorModel = this.bpclEkycService.getBpclOperatorByUid(operatorID);
        operatorRefillRequestData.setPartnerCode(this.getConfigurationService().getConfiguration().getString("operator.partner.code"));
        operatorRefillRequestData.setPartnerTransId(this.getOperatorRefillKeyGenerator().generate().toString());
        operatorRefillRequestData.setUniqueAPIReqNo(this.operatorUniqueApiKeyGenerator.generate().toString());
        operatorRefillRequestData.setAddl1(operatorModel.getStaffId());
        operatorRefillRequestData.setEndPoint("operatorRefillBookingUrl");
        if (StringUtils.isNotBlank(operatorRefillRequestData.getConsumerNo())) {
            operatorRefillRequestData.setDistCode(operatorModel.getDistributorId());
        }
        final String hashValue = this.getHashKeyValueRequest(operatorRefillRequestData);
        operatorRefillRequestData.setHashKey(hashValue);
    }
    
    private String getHashKeyValueRequest(final LpgRefillBookingRequestData requestData) {
        final StringBuilder hashMessage = new StringBuilder();
        final String endPointName = requestData.getEndPoint();
        final String partnerCode = this.lpgCredentialsService.getPartnerCodeByApiName(endPointName);
        final String secret = this.lpgCredentialsService.getPartnerSecretByApiName(endPointName);
        hashMessage.append(partnerCode);
        hashMessage.append(requestData.getPartnerTransId());
        hashMessage.append(requestData.getUniqueAPIReqNo());
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumberType()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getContactNumber()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getLpgId()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getDistCode()));
        hashMessage.append(this.getEmptyIfBlank(requestData.getConsumerNo()));
        hashMessage.append("");
        hashMessage.append("");
        hashMessage.append("");
        hashMessage.append("");
        hashMessage.append(requestData.getAddl1());
        hashMessage.append("");
        hashMessage.append("");
        hashMessage.append(secret);
        final String encryptedMsg = this.getLpgEncryptionService().encodeBySHA512(hashMessage.toString());
        return encryptedMsg;
    }
    
    private String getEmptyIfBlank(final String stringToCheck) {
        return org.apache.commons.lang.StringUtils.isBlank(stringToCheck) ? "" : stringToCheck;
    }
    
    public HttpResponse<String> submitRefillBookingRequest(final LpgRefillBookingRequestData refillRequestData, final OperatorCODRefillsModel operatorCodRefillBookingModel) throws JsonProcessingException {
        final String refillDetailsUrl = MessageFormat.format(this.configurationService.getConfiguration().getString(refillRequestData.getEndPoint()), refillRequestData.getUniqueAPIReqNo());
        Objects.requireNonNull(refillDetailsUrl, "Endpoint should not be null");
        return this.getBpclHttpClientService().executePostMethodWithTimeOut(refillDetailsUrl, this.getRequestJsonForOperator(refillRequestData), refillRequestData.getEndPoint(), operatorCodRefillBookingModel);
    }
    
    private String getRequestJsonForOperator(final LpgRefillBookingRequestData refillRequestData) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(refillRequestData);
        if (LpgOperatorServiceImpl.LOG.isDebugEnabled()) {
            LpgOperatorServiceImpl.LOG.debug(MessageFormat.format("Input json :: {0} for end point :: {1} ", json, refillRequestData.getEndPoint()));
        }
        return objectMapper.writeValueAsString(refillRequestData);
    }
    
    public void setModelService(final ModelService modelService) {
        this.modelService = modelService;
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setLpgCredentialsService(final LPGCredentialsService lpgCredentialsService) {
        this.lpgCredentialsService = lpgCredentialsService;
    }
    
    public BpclHttpClientService getBpclHttpClientService() {
        return this.bpclHttpClientService;
    }
    
    public void setBpclHttpClientService(final BpclHttpClientService bpclHttpClientService) {
        this.bpclHttpClientService = bpclHttpClientService;
    }
    
    public LpgEncryptionService getLpgEncryptionService() {
        return this.lpgEncryptionService;
    }
    
    public void setLpgEncryptionService(final LpgEncryptionService lpgEncryptionService) {
        this.lpgEncryptionService = lpgEncryptionService;
    }
    
    public KeyGenerator getOperatorRefillKeyGenerator() {
        return this.operatorRefillKeyGenerator;
    }
    
    public void setOperatorRefillKeyGenerator(final KeyGenerator operatorRefillKeyGenerator) {
        this.operatorRefillKeyGenerator = operatorRefillKeyGenerator;
    }
    
    public void setBpclEkycService(final BpclEkycService bpclEkycService) {
        this.bpclEkycService = bpclEkycService;
    }
    
    static {
        LOG = LogManager.getLogger((Class)LpgOperatorServiceImpl.class);
    }
}
